/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.bpmn.views.data {
  import com.adobe.cairngorm.CairngormError;
  import com.adobe.cairngorm.CairngormMessageCodes;

  import flash.display.DisplayObject;
  import flash.events.Event;
  import flash.geom.Rectangle;

  import mx.binding.utils.BindingUtils;
  import mx.binding.utils.ChangeWatcher;
  import mx.containers.Canvas;
  import mx.containers.HBox;
  import mx.containers.VBox;
  import mx.controls.Image;
  import mx.controls.Label;
  import mx.controls.Spacer;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.data.AbstractExpressionElementModel;
  import org.ow2.orchestra.designer.bpmn.views.*;

  public class AbstractExpressionElementView extends AbstractElementView implements IFlowNodeView {
    [Bindable]
    private var _dataModel:AbstractExpressionElementModel;

    private var _imageDataBackground:Image = new Image();

    protected var _label:Label;

    function AbstractExpressionElementView(dataModel:AbstractExpressionElementModel) {
      _dataModel = dataModel;
      this.setStyle("horizontalAlign", "center");
      _imageDataBackground.height = 30;
      _imageDataBackground.width = 20;
      _imageDataBackground.automationName = "DataImage" + _dataModel.name;

      watchers.addItem(BindingUtils.bindProperty(this, "x", _dataModel, "x"));
      watchers.addItem(BindingUtils.bindProperty(this, "y", _dataModel, "y"));

      _label = new Label();

      var hBox:HBox = new HBox();
      hBox.percentWidth = 100;
      var sp1:Spacer = new Spacer();
      sp1.percentWidth = 100;
      var sp2:Spacer = new Spacer();
      sp2.percentWidth = 100;
      hBox.addChild(sp1);
      hBox.addChild(_imageDataBackground);
      hBox.addChild(sp2);
      this.addChild(hBox);
      this.addChild(_label);

      this.updateImage();
    }


    override public function get model():AbstractElement {
      return _dataModel;
    }

    override public function copy():AbstractElementView {
      return new AbstractExpressionElementView(_dataModel);
    }

    override public function get borderImage():DisplayObject {
      return _imageDataBackground;
    }

    override public function get subtypeImage():DisplayObject {
      return null;
    }


    public function getCollisionRectangle(targetCoordinateSpace:DisplayObject):Rectangle {
      return this.borderImage.getRect(targetCoordinateSpace);
    }


    protected function get imageDataBackground():Image {
      return _imageDataBackground;
    }

    protected function updateImage(event:Event = null):void {
      //This method is designed to be abstract
      throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
    }
  }
}
