/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.bpmn.views.data {
import flash.display.DisplayObject;
import flash.events.Event;
import flash.geom.Rectangle;

import mx.binding.utils.BindingUtils;
import mx.binding.utils.ChangeWatcher;
import mx.containers.Canvas;
import mx.containers.HBox;
import mx.containers.VBox;
import mx.controls.Image;
import mx.controls.Label;
import mx.controls.Spacer;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionOperationModel;
import org.ow2.orchestra.designer.bpmn.model.data.OperationType;
import org.ow2.orchestra.designer.bpmn.views.*;

public class ExpressionOperationView extends AbstractElementView implements IFlowNodeView {
  [Bindable]
  private var _expressionOperationModel:ExpressionOperationModel;

  /**
   * The source for "event" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/gateway.svg")]
  private static var _image:Class;

  private var _imageDataBackground:Image = new Image();

  private var _label:Label;

  function ExpressionOperationView(dataModel:ExpressionOperationModel) {
    _expressionOperationModel = dataModel;
    this.setStyle("horizontalAlign", "center");
    _imageDataBackground.automationName = "OperationImage" + _expressionOperationModel.name;
    _imageDataBackground.source = _image;
    _imageDataBackground.height = 40;
    _imageDataBackground.width = 40;

    watchers.addItem(BindingUtils.bindProperty(this, "x", _expressionOperationModel, "x"));
    watchers.addItem(BindingUtils.bindProperty(this, "y", _expressionOperationModel, "y"));
    watchers.addItem(ChangeWatcher.watch(_expressionOperationModel, ["type"], this.updateType));

    _label = new Label();

    var canvas:Canvas = new Canvas();

    var hBox1:HBox = new HBox();
    hBox1.percentWidth = 100;
    var s1:Spacer = new Spacer();
    s1.percentWidth = 100;
    var s2:Spacer = new Spacer();
    s2.percentWidth = 100;
    hBox1.addChild(s1);
    hBox1.addChild(_imageDataBackground);
    hBox1.addChild(s2);
    canvas.addChild(hBox1);

    var hbox:HBox = new HBox();
    hbox.percentWidth = 100;
    hbox.percentHeight = 100;
    var vbox:VBox = new VBox();
    vbox.percentHeight = 100;
    vbox.percentWidth = 100;
    var s3:Spacer = new Spacer();
    s3.percentHeight = 100;
    var s4:Spacer = new Spacer();
    s4.percentHeight = 100;
    var s5:Spacer = new Spacer();
    s5.percentWidth = 100;
    var s6:Spacer = new Spacer();
    s6.percentWidth = 100;

    vbox.addChild(s3);
    vbox.addChild(_label);
    vbox.addChild(s4);
    hbox.addChild(s5);
    hbox.addChild(vbox);
    hbox.addChild(s6);

    canvas.addChild(hbox);
    this.addChild(canvas);


    this.updateType();
  }

  private function updateType(event:Event = null):void {
    var type:OperationType = _expressionOperationModel.type;
    this._label.text = getTypeLabel(type);
  }

  public static function getTypeLabel(type:OperationType):String {
    if (type != null) {
      switch (type) {
        case OperationType.CONCATENATE:
        {
          return "\"ab\"";
        }
        case OperationType.ADD:
        {
          return "a + b";
        }
        case OperationType.SUBTRACT:
        {
          return "a - b";
        }
        case OperationType.DIVIDE:
        {
          return "a / b";
        }
        case OperationType.MULTIPLY:
        {
          return "a * b";
        }
        case OperationType.IS_GREATER_THAN:
        {
          return "a > b";
        }
        case OperationType.IS_LESS_THAN:
        {
          return "a < b";
        }
        case OperationType.IS_GREATER_OR_EQUALS:
        {
          return "a ≥ b";
        }
        case OperationType.IS_LESS_OR_EQUALS:
        {
          return "a ≤ b";
        }
        case OperationType.EQUALS:
        {
          return "a = b";
        }
        case OperationType.NOT_EQUALS:
        {
          return "a ≠ b";
        }
        case OperationType.OR:
        {
          return " OR ";
        }
        case OperationType.AND:
        {
          return " AND ";
        }
        default :
        {
          return "";
        }
      }
    } else {
      return "";
    }
  }

  override public function get model():AbstractElement {
    return _expressionOperationModel;
  }

  override public function copy():AbstractElementView {
    return new ExpressionOperationView(_expressionOperationModel);
  }

  override public function get borderImage():DisplayObject {
    return _imageDataBackground;
  }

  override public function get subtypeImage():DisplayObject {
    return null;
  }


  public function getCollisionRectangle(targetCoordinateSpace:DisplayObject):Rectangle {
    return this.borderImage.getRect(targetCoordinateSpace);
  }


  protected function get imageDataBackground():Image {
    return _imageDataBackground;
  }

}
}
