/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.bpmn.views.data {
import flash.display.DisplayObject;
import flash.events.Event;
import flash.geom.Rectangle;

import mx.binding.utils.BindingUtils;
import mx.binding.utils.ChangeWatcher;
import mx.containers.HBox;
import mx.controls.Image;
import mx.controls.Label;
import mx.controls.Spacer;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.LiteralModel;
import org.ow2.orchestra.designer.bpmn.views.*;

public class LiteralView extends AbstractElementView implements IFlowNodeView {
  [Bindable]
  private var _literalModel:LiteralModel;

  /**
   * The source for "event" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/literal.svg")]
  private static var literalImage:Class;

  private var _imageDataBackground:Image = new Image();

  private var _label:Label;

  function LiteralView(dataModel:LiteralModel) {
    _literalModel = dataModel;
    this.setStyle("horizontalAlign", "center");
    _imageDataBackground.height = 30;
    _imageDataBackground.width = 20;
    _imageDataBackground.automationName = "LiteralImage" + _literalModel.name;
    _imageDataBackground.source = literalImage;

    watchers.addItem(BindingUtils.bindProperty(this, "x", _literalModel, "x"));
    watchers.addItem(BindingUtils.bindProperty(this, "y", _literalModel, "y"));
    watchers.addItem(ChangeWatcher.watch(_literalModel, ["value"], this.updateText));

    _label = new Label();

    var hBox:HBox = new HBox();
    hBox.percentWidth = 100;
    var sp1:Spacer = new Spacer();
    sp1.percentWidth = 100;
    var sp2:Spacer = new Spacer();
    sp2.percentWidth = 100;
    hBox.addChild(sp1);
    hBox.addChild(_imageDataBackground);
    hBox.addChild(sp2);
    this.addChild(hBox);
    this.addChild(_label);

    this.updateText();
  }

  private function updateText(event:Event = null):void {
    if (_literalModel.value != null) {
      this._label.text = _literalModel.value;
    } else {
      this._label.text = "";
    }
  }

  override public function get model():AbstractElement {
    return _literalModel;
  }

  override public function copy():AbstractElementView {
    return new LiteralView(_literalModel);
  }

  override public function get borderImage():DisplayObject {
    return _imageDataBackground;
  }

  override public function get subtypeImage():DisplayObject {
    return null;
  }


  public function getCollisionRectangle(targetCoordinateSpace:DisplayObject):Rectangle {
    return this.borderImage.getRect(targetCoordinateSpace);
  }


  protected function get imageDataBackground():Image {
    return _imageDataBackground;
  }

}
}
